//Erweiterung von Klasse MouseBehavior mit eingeschrnktem Translationsverhalten in X-Richtung

//KOMMENTARE:
//U.U. manche Fallunterscheidungen weglassen(invert, manualwakeup, reset, ...)
//WICHTIG: Falls man weiss, dass das Behavior an keinem allgemeinen Trafo-Knoten hngt,
//also sicher ist, dass der Trafo-Knoten keine Translationskomponente hat,
//z.B. gibt es einen Extra-Translationsknoten, dann knnte
//man die processStimulus-Funktion etwas vereinfachen!
//Aufruf mit:
//   MouseTranslateXLimit behavior = new MouseTranslateXLimit();
//   behavior.setTransformGroup(objTrans);
//   objTrans.addChild(behavior);
//   behavior.setSchedulingBounds(bounds);
//Beachte auch die Felder in MouseBehavior:
//flags, invert, INVERT_INPUT, MANUAL_WAKEUP, wakeup, reset
//buttonPress, mouseCriterion, mouseEvents
//transformGroup, currXform, transformX,m transformY,
//x, y, x_last, y_last   

//TODO: Invert
//TODO: Kommentare weg oder verenglischen

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.mouse.*;

public class MouseTranslateXLimit extends MouseBehavior {

   //Felder
   double x_value;//Aktueller Differenz-Wert
   double x_factor;//Stepwinkel
   double x_min;
   double x_max;

   //Vector3d translation = new Vector3d();//Alternativ
   Matrix4d mat = new Matrix4d();
		    
   private MouseBehaviorCallback callback = null;//NEU

   //Erstellt MouseBehavior zu vorgegebener Transformationsgruppe
   public MouseTranslateXLimit(TransformGroup transformGroup, double x_left, double x_right) {
     super(transformGroup);
     x_min = x_left;
     x_max = x_right;
     x_factor = .02;      
   }

   //Default-Constructor
   public MouseTranslateXLimit() {
     super(0);
     x_min = 0;
     x_max = 0;
     x_factor = .02;      
   }  
   
   //Erstellt MouseBehavior mit Flags
   public MouseTranslateXLimit(int flags) {
      super(flags);
      x_min = 0;
      x_max = 0;
      x_factor = .02;      
   }   

   //Erweitere "initialize"
   public void initialize() {
    super.initialize();
    x_value = 0;
    if ((flags & INVERT_INPUT) == INVERT_INPUT) {
       invert = true;                            
       x_factor *= -1;
    }
  }

  //Gib Faktor zurck
  public double getXFactor() {
    return x_factor;
  }
  
  //Setze Faktor
  public void setXFactor( double factor) {
    x_factor = factor;
  }
 
  //Get und Set Funktionen fr linken und rechten Rand
   
  public double getXMin() {
     return x_min;
  }
   
  public double getXMax() {
     return x_max;
  }

  public void setXMin(double x_left) {
     x_min = x_left;
  }

  public void setXMax(double x_right) {
     x_max = x_right;
  }

  //Funktion, die nach Aktivierung jeweils ausgefhrt wird
   public void processStimulus (Enumeration criteria) {
      WakeupCriterion wakeup;
      AWTEvent[] event;
      int id;
      int dx;

      //Gehe alle Wakeup-Kriterien der Reihe nach durch
      while (criteria.hasMoreElements()) {
         wakeup = (WakeupCriterion) criteria.nextElement();

	 //AWT-Ereignis
         if (wakeup instanceof WakeupOnAWTEvent) {
            event = ((WakeupOnAWTEvent)wakeup).getAWTEvent();//Hole AWT-Event

	    //Gehe alle AWT-Events der Reihe nach durch und rufe Mouse-Handling auf
            for (int i=0; i<event.length; i++) { 
	      processMouseEvent((MouseEvent) event[i]);

	      //(1+) Falls Knopfdruck oder Manual-WakeUp, (1-) sonst egal
	      if (((buttonPress)&&((flags & MANUAL_WAKEUP) == 0)) ||
		  ((wakeUp)&&((flags & MANUAL_WAKEUP) != 0))){
		
		id = event[i].getID();
		//(2+) Falls Maus nicht mit 2.Taste oder mit gedrckter ALT-Taste bewegt wurde
		if ((id == MouseEvent.MOUSE_DRAGGED) && 
		    !((MouseEvent)event[i]).isMetaDown() && 
		    !((MouseEvent)event[i]).isAltDown()){

		  //Berechne jetzt neue Differenz im x-Anteil der Mausbewegung
                  x = ((MouseEvent)event[i]).getX();
                  dx = x - x_last;
		   
		  //(3+) Falls kein Reset
		  if (!reset){

		    transformGroup.getTransform(currXform);

                    //Berechne neuen x-Differenzwert und entsprechende Translationsmatrix
		    x_value = dx * x_factor;
 		    currXform.get(mat);
		    double x_new = mat.m03 + x_value;
	            if ((x_new >= x_min) && (x_new <= x_max))
   		      currXform.setTranslation(new Vector3d(x_new, mat.m13, mat.m23));

                    //Alternativ: TODO Originalcode aus MouseTranslate
		    //(Besser? Eine ganze Matrizenmultiplikation? Halt uebersichtlicher!
                    //Oder lieber wieder alt, ohne Translationfeld, aber mit mat Feld?)
		    //translation.x = dx * x_factor;
		    //
		    //if ((translation.x >= x_min) && (translation.x <= x_max))
		    //{
		    // transformX.set(translation);
		    //
		    // if (invert) {
		    //   currXform.mul(currXform, transformX);
		    // } else {
		    //   currXform.mul(transformX, currXform);
		    // }
		    //}
		
		    transformGroup.setTransform(currXform);

		    //NEU:
		    transformChanged( currXform );
                    if (callback!=null)
                        callback.transformChanged( MouseBehaviorCallback.TRANSLATE, currXform );

		  }
		  //(3-) Falls Reset
		  else {
		    reset = false;
		  }

	          //(2...) Aktualisiere x_last
                  x_last = x;
               }
	       //(2-) Falls Maus nicht bewegt wurde oder mit 2.Taste, bzw. ALT
               else if (id == MouseEvent.MOUSE_PRESSED) {
                  x_last = ((MouseEvent)event[i]).getX();
               }
	      }
	    }
         }
      }

      //Setze nchstes Wakeup-Criterion fest
      wakeupOn (mouseCriterion);
   }

  /**
    * Users can overload this method  which is called every time
    * the Behavior updates the transform
    *
    * Default implementation does nothing
    */
  public void transformChanged( Transform3D transform ) {
  }

  /**
    * The transformChanged method in the callback class will
    * be called every time the transform is updated
    */
  public void setupCallback( MouseBehaviorCallback callback ) {
      this.callback = callback;
  }
}
